#![feature(proc_macro_hygiene, decl_macro)]

#[macro_use] extern crate rocket;

use std::result::Result;
use std::fs;
use std::io;
use rocket::request::{Request, FromParam};
use rocket::http::RawStr;
use rocket_contrib::templates::Template;

const CONTENT_DIR:&str = "content";

#[derive(serde::Serialize, Debug)]
struct Content {
    title: String,
    content: String,
}

impl<'r> FromParam<'r> for Content {
    type Error = &'r RawStr;

    fn from_param(param: &'r RawStr) -> Result<Self, Self::Error> {
        let filename = format!("{}/{}.md", CONTENT_DIR, param);
        let content = fs::read_to_string(&filename);
        match content {
            Ok(value) => Ok(Content{ title: param.to_string(), content: value }),
            Err(_) => Err(param),
        }
    }
}

fn list_contents() -> io::Result<Vec<String>> {
    let mut contents: Vec<String> = vec!();
    for path in fs::read_dir(CONTENT_DIR)? {
        let file = path?;
        let content = format!("{:?}", file.file_name()).replace("\"", "").replace(".md", "");
        contents.push(content);
    }
    Ok(contents)
}

#[derive(serde::Serialize)]
struct IndexContext {
    title: String,
    items: Vec<String>
}

#[catch(404)]
fn not_found(req: &Request<'_>) -> Template {
    let dbg = format!("{:#?}", req);
    let mut map = std::collections::HashMap::new();
    map.insert("title", "Page not found");
    map.insert("path", req.uri().path());
    map.insert("req", &dbg);
    Template::render("error/404", &map)
}

#[get("/<content>")]
fn show(content: Content) -> Template {
    Template::render("content", &content)
}

#[get("/")]
fn index() -> io::Result<Template> {
    let cwd = std::env::current_dir()?;
    let title = String::from(cwd.to_str().unwrap().split("/").last().unwrap());
    //println!("{}", title);
    let contents = list_contents()?;
    let context = IndexContext { title: title, items: contents };
    Ok(Template::render("index", &context))
}

fn main() {
    rocket::ignite()
        .attach(Template::fairing())
        .mount("/", routes![index, show])
        .register(catchers![not_found])
        .launch();
}

